function afemDPG(example,method,theta,minNrDoF)
% Function for running benchmarks for dPG methods
% Input:   example    problem to solve, for options see below
%          method     which dPG method, for options see below
%          theta      bulk parameter, uniform refinement if set to 1
%          minNrDoF   number of degrees of freedom to be reached minimally
    
    % Add current path
    addpath(genpath(pwd));

    %% Set data if none is given
    if( nargin == 0)
        % Options for problem
%         example = 'LshapeDirac';        
%         example = 'LshapeDiracReverse';        
s%         example = 'LshapeExact';        
        example = 'OneRHS';        
%         example = 'SlitExact';        
%         example = 'SquareExact';        
%         example = 'SquareExact2';        
%         example = 'SquareExact3';        
%         example = 'Waterfall';       

        method = 'primal';
%         method = 'ultraweak';

        % Set Parameters
        theta = 1;
        minNrDoF = 10000; 
    end

    %% Collect data for example
    data = eval(example);
    c4n = data{1};
    n4e = data{2};
    n4sDb = data{3};
    n4sNb = data{4};
    f = data{5};
    u4Db = data{6};
    g = data{7};
    exSolKnown = data{8};
    % optional exact solution data
    if exSolKnown == 1
        uExact = data{9};
        graduExact = data{10}; 
    end 
    
    %% Output of data
    % Set name for folder in which to save data
    if theta == 1
        folder = ['output/dpg_',method,'_',example,'_minDoF',num2str(minNrDoF)];
    else
        folder = ['output/dpg_',method,'_',example,'_minDoF',num2str(minNrDoF),'_theta',num2str(theta)];
    end
    % Make folder
    if ~exist(folder,'dir')
        mkdir(folder);
    end
    
    % Set number of nodes up to which mesh data should be saved
    triangBound = 10000;
    
    % Initialize data vectors
    nrDoF4lvl = [];
    eta4lvl = [];
    data4lvl = [];
    error4lvlTotal = [];
    error4lvlR = [];
    error4lvlW = [];
    error4lvlT = [];
    error4lvlU = [];
    error4lvlQ = [];
    error4lvlV = [];
    
    %% Start AFEM loop SOLVE -> ESTIMATE -> MARK -> REFINE
    while(true)
        %%%%%%%%%%%%
        % SOLVE
        %%%%%%%%%%%%
        
        if strcmp(method,'primal')
            [u,t,v, nrDoF, data] = solveDPGPrimal(c4n,n4e,n4sDb,n4sNb,f,u4Db,g);       
        elseif strcmp(method,'ultraweak')
            [r,w,t,u,q,v, nrDoF, data] = solveDPGUltraweak(c4n,n4e,n4sDb,n4sNb,f,u4Db,g);
        end
       
        nrDoF4lvl(end+1) = nrDoF;
        data4lvl = [data4lvl; data];
                    
        %%%%%%%%%%%%
        % ESTIMATE
        %%%%%%%%%%%%
        if strcmp(method,'primal')
            [eta4e,error4eV] = estimatePrimalDPG(c4n,n4e,f,v); 
        elseif strcmp(method,'ultraweak')
            [eta4e,error4eQ,error4eV] = estimateUltraweakDPG(c4n,n4e,f,q,v); 
        end
        % Collect estimator in vector
        eta4lvl(end+1) = sqrt(sum(eta4e));
        
        %%%%%%%%%%%%if (compare && alpha==1 && strcmp(Q,'identity'))
        % (Errors)
        %%%%%%%%%%%%
        % Compute errors if exact solution is known
        if exSolKnown == 1
            if strcmp(method,'primal')
                [error4eU,error4eT] = error4ePrimalDPG(c4n,n4e,f,graduExact,u,t);
                % Collect errors in vectors
                error4lvlTotal(end+1) = sqrt(sum(error4eU+error4eT+error4eV));
                error4lvlU(end+1) = sqrt(sum(error4eU));  
                error4lvlT(end+1) = sqrt(sum(error4eT));              
                error4lvlV(end+1) = sqrt(sum(error4eV));
                % save data
                fileIDDPG = fopen([folder,'/convergence.dat'],'w');
                fprintf(fileIDDPG,'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n','nrDoF4lvl','error4lvlU','error4lvlT','error4lvlV','error4lvlTotal','eta4lvl', 'assemble', 'solve', 'cond');
                dlmwrite([folder,'/convergence.dat'], [nrDoF4lvl',error4lvlU',error4lvlT',error4lvlV',error4lvlTotal',eta4lvl',data4lvl],'delimiter', '\t', '-append')
                fclose(fileIDDPG);
                close all; 
            elseif strcmp(method,'ultraweak')
                [error4eR, error4eW, error4eT,error4eU] = error4eUltraweakDPG(c4n,n4e,f,uExact,graduExact,r,w,t,u);
                % Collect errors in vectors
                error4lvlTotal(end+1) = sqrt(sum(error4eR+error4eW+error4eT+error4eU+error4eQ+error4eV));
                error4lvlR(end+1) = sqrt(sum(error4eR)); 
                error4lvlW(end+1) = sqrt(sum(error4eU)); 
                error4lvlT(end+1) = sqrt(sum(error4eT)); 
                error4lvlU(end+1) = sqrt(sum(error4eU)); 
                error4lvlQ(end+1) = sqrt(sum(error4eU)); 
                error4lvlV(end+1) = sqrt(sum(error4eU)); 
                fileIDDPG = fopen([folder,'/convergence.dat'],'w');
                fprintf(fileIDDPG,'%s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\t %s\n','nrDoF4lvl','error4lvlR','error4lvlW','error4lvlT','error4lvlU','error4lvlQ','error4lvlV','error4lvlTotal','eta4lvl', 'assemble', 'solve', 'cond');
                dlmwrite([folder,'/convergence.dat'], [nrDoF4lvl',error4lvlR',error4lvlW',error4lvlT',error4lvlU',error4lvlQ',error4lvlV',error4lvlTotal',eta4lvl',data4lvl],'delimiter', '\t', '-append')
                fclose(fileIDDPG);
                close all; 
                
            end    
        else
            fileIDDPG = fopen([folder,'/convergence.dat'],'w');
            fprintf(fileIDDPG,'%s\t %s\t %s\t %s\t %s\n','nrDoF4lvl','eta4lvl', 'assemble', 'solve', 'cond');
            dlmwrite([folder,'/convergence.dat'], [nrDoF4lvl',eta4lvl',data4lvl],'delimiter', '\t', '-append')
            fclose(fileIDDPG);
        end    
            
        % Output in terminal
        disp(['nodes/dofs: ',num2str(size(c4n,1)),'/',num2str(nrDoF),...
             '; estimator = ',num2str(eta4lvl(end))]); 

        % Quit loop if minimal desired number of dof is reached
        if nrDoF >= minNrDoF 
            break;
        end
            
        %%%%%%%%%%%%%%%%%%%
        % MARK and REFINE
        %%%%%%%%%%%%%%%%%%%
        
        % Collective Marking algorithm
        if theta == 1
            % Compute new triangulation via uniform red refinement
            [c4n,n4e,n4sDb,n4sNb] = refineUniformRed(c4n,n4e,n4sDb,n4sNb);     
        else
            % Mark elements via bulk criterion
            n4sMarked = markBulk(n4e,eta4e,theta);
            % Compute new triangulation via Bi3/Green/Blue refinement
            [c4n,n4e,n4sDb,n4sNb] = refineBi3GB(c4n,n4e,n4sDb,n4sNb,n4sMarked);
        end
    end

    % For raw meshes, save mesh data
    if( size(c4n,1) <= triangBound)
    	fileID1 = fopen([folder,'/c4n.dat'],'w');
    	fprintf(fileID1,'%s\t  %s\n','x','y');
    	dlmwrite([folder,'/c4n.dat'], c4n,'delimiter', '\t', '-append')
    	fclose(fileID1);
    
    	fileID2 = fopen([folder,'/n4e.dat'],'w');
    	fprintf(fileID2,'%s\t  %s\t %s\n','P1','P2','P3');
    	dlmwrite([folder,'/n4e.dat'], n4e-1,'delimiter', '\t', '-append')
    	fclose(fileID2);
    end
    
    % save convergence history data and plot it
    if exSolKnown == 1
    % for known exact solution, errors can be plotted, too        
        % plot convergence to figure
        figure(1);
        plotConvergenceColor(nrDoF4lvl,error4lvlTotal,'||x - x_h||');
        hold on;
        plotConvergenceColor(nrDoF4lvl,error4lvlU,'||u - u_C||');
        plotConvergenceColor(nrDoF4lvl,error4lvlT,'||t - t_0||');
        plotConvergenceColor(nrDoF4lvl,error4lvlV,'||v_{CR}||');
        plotConvergenceColor(nrDoF4lvl,eta4lvl,['eta']);
        if strcmp(method,'ultraweak')
            plotConvergenceColor(nrDoF4lvl,error4lvlR,'||\nabla u - r_0||');
            plotConvergenceColor(nrDoF4lvl,error4lvlW,'||u - w_0||');
            plotConvergenceColor(nrDoF4lvl,error4lvlQ,'||q_1||');
        end
    else
    % if no exact solution is known, plot only estimator      
        % plot convergence to figure
        figure(1);
        plotConvergenceColor(nrDoF4lvl,eta4lvl,['eta']);
        hold on;
    end
    
    % save convergence history plot to file
    print( figure(1), '-dpng', [folder,'/ConvergencePlot.png']);

    
    
    % for raw meshes, plot deformation and save triangulation to file
    if ( size(c4n,1)<=triangBound )
         %plot u
         figure(2);
         plotP1(c4n, n4e, u(:), 'P1 Solution');
         
         % plot v
         figure(3);
         plotCR(c4n, n4e, v(:), 'Riesz Representation');

         if exSolKnown == 1
             uCompare = uExact(c4n);
             figure(4);
             plotP1(c4n, n4e, uCompare, 'Exact Solution');
         end
        
        
         % save triangulation to file
         figure(5);
         plotTriangulation(c4n,n4e);
         print( figure(2), '-dpng', [folder,'/Triangulation.png']);

         % save solution u to file
         fileID3 = fopen([folder,'/u.dat'],'w');
         fprintf(fileID3,'%s\t %s\t %s\n','x','y','u');
         dlmwrite([folder,'/u.dat'],[c4n(n4e',:),reshape(u(n4e'),3*size(n4e,1),1)],'delimiter','\t','-append');
         fclose(fileID3);
         
         % save solution v to file
         fileID4 = fopen([folder,'/v.dat'],'w');
         dlmwrite ( [folder,'/v.dat'],[c4n(n4e',:),v],'delimiter','\t','newline','unix');
         fclose(fileID4);
         
         tCompare = zeros(size(n4e,1),2);
         n4s = computeN4s(n4e);
         area4e = computeArea4e(c4n,n4e);
         mid4e = computeMid4e(c4n, n4e);
         length4s = computeLength4s(c4n,n4s);
         normal4e = computeNormal4e(c4n,n4e);
         normal4s = computeNormal4s(c4n,n4s);
         s4e = computeS4e(n4e);
         for curElem = 1 : size(n4e,1)
            % Compute the coordinates of the nodes of the current element
            coords = c4n(n4e(curElem,:),:);
            % Compute the sides of the current element in global enumeration of
            % sides as in n4s
            sides = s4e(curElem,:);
            signedLength = sum(normal4s(sides,:)'.*normal4e(:,:,curElem)') .* length4s(sides,:)';
            tCompare(curElem,:) =sum(repmat(t(sides,:).*signedLength',1,2).*(repmat(mid4e(curElem,:),3,1)-coords([3 1 2],:)))/(2*area4e(curElem));
         end
         % save solution t to file (quiver plot)
         fileID6 = fopen([folder,'/t.dat'],'w');
         fprintf(fileID6,'%s\t %s\t %s\t %s\t %s\n','x','y','r1','r2','z');
         dlmwrite([folder,'/t.dat'],[mid4e,tCompare,sqrt(sum(tCompare.^2,2))],'delimiter','\t','-append');
         fclose(fileID6); 
         
         % for ultraweak method, save also piecewise constant parts of the
         % solutions to file
         if strcmp(method,'ultraweak')
             % plot w
             figure(6);
             plotP04e(c4n, n4e, w(:), 'Riesz Representation');
             
             % save solution w to file
             fileID5 = fopen([folder,'/w.dat'],'w');
             fprintf(fileID5,'%s\t %s\t %s\n','x','y','w');
             dlmwrite([folder,'/w.dat'],[c4n(n4e',:),reshape(repmat(w',3,1),3*size(n4e,1),1)],'delimiter','\t','-append');
             fclose(fileID5);
             
             mid4e = computeMid4e(c4n, n4e);
             % save solution r to file (quiver plot)
             fileID6 = fopen([folder,'/r.dat'],'w');
             fprintf(fileID6,'%s\t %s\t %s\t %s\t %s\n','x','y','r1','r2','z');
             dlmwrite([folder,'/r.dat'],[mid4e,r,sqrt(sum(r.^2,2))],'delimiter','\t','-append');
             fclose(fileID6);
             
             % save solution q to file (quiver plot)
             qP0 = q(:,[1 2]);
             fileID6 = fopen([folder,'/q.dat'],'w');
             fprintf(fileID6,'%s\t %s\t %s\t %s\t %s\n','x','y','r1','r2','z');
             dlmwrite([folder,'/q.dat'],[mid4e,qP0,sqrt(sum(qP0.^2,2))],'delimiter','\t','-append');
             fclose(fileID6);      
         end
    end
end



% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
