%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% L-shaped domain with Exact Solution %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = LshapeExact
    exSolKnown = 1;
    [c4n,n4e,n4sDb,n4sNb] = loadGeometry('LshapeNb',1);    
    data = {c4n,n4e,n4sDb,n4sNb,@f,@u4Db,@g,exSolKnown,...
                    @uExact,@graduExact}; 
end

%% PROBLEM INPUT DATA
% Right-hand side function
function val = f(x)
   val = zeros(size(x,1),1);
end

% Dirichlet boundary data
function val = u4Db(x)
    [phi, r] = cart2pol(x(:,1),x(:,2));
    [index] = find(phi<0);
    phi(index) = phi(index) + 2*pi;
    val = r.^(2/3).*sin(2/3*phi);
end

% Exact solution
function val = uExact(x)
    [phi, r] = cart2pol(x(:,1),x(:,2));
    [index] = find(phi<0);
    phi(index) = phi(index) + 2*pi;
    val = r.^(2/3).*sin(2/3*phi);
end

% Gradient of exact solution
function val = graduExact(x)
    [phi, r] = cart2pol(x(:,1),x(:,2));
    [index] = find(phi<0);
    phi(index) = phi(index)+2*pi;
    if ~isempty( find(phi<0)) && ~isempty(find(phi>2*pi))
        error('winkel')
    end
    val(:,1) = 2/3*r.^(-1/3).*cos(2/3*phi);
    val(:,2) = 2/3*r.^(-1/3).*sin(2/3*phi);
    for i = 1 : (size(x,1))
        if r(i)==0
            val(i,2) = Inf;
        end
        val(i,:)=val(i,:)*[-sin(phi(i)), cos(phi(i));...
            cos(phi(i)), sin(phi(i))];
    end
end  

% Neumann boundary data
function val = g(x)
    x1 = x(:,1);
    x2 = x(:,2);
    sigma = graduExact(x);
    for i=1:(size(x,1))    
        if x1(i)==0 || x1(i) == 1
            N = [1;0];
        elseif  x1(i)== - 1
            N = [- 1;0];  
        elseif x2(i)==-1 || x2(i)== 0
            N = [0;-1];
        elseif x2(i)==1
            N = [0;1];     
        end    
        val(i,:) = sigma(i,:)*N;
    end 
end

% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.