%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Square domain with exact solution %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = SquareExact
    exSolKnown = 1;
    [c4n,n4e,n4sDb,n4sNb] = loadGeometry('Square',1);
    data = {c4n,n4e,n4sDb,n4sNb,@f,@u4Db,@g,exSolKnown,...
                    @uExact,@graduExact};
end

%% PROBLEM INPUT DATA
% Right-hand side function
function val = f(x)
    val = -2*(x(:,2).*(x(:,2)-1)+ x(:,1).*(x(:,1)-1));
end

% Dirichlet boundary data
function val = u4Db(x)
    val = zeros(size(x,1),1);
end

% Exact solution
function val = uExact(x)
    val = x(:,2).*(x(:,2)-1).* x(:,1).*(x(:,1)-1);
end

% Gradient of exact solution
function val = graduExact(x)
    val = [(2*x(:,1)-1).* x(:,2).*(x(:,2)-1),...
            (2*x(:,2)-1).* x(:,1).*(x(:,1)-1)];
end

% Neumann boundary data
function val = g(x)
    x1 = x(:,1);
    x2 = x(:,2);
    sigma = graduExact(x);
    for i=1:(size(x,1))    
        if x1(i)==0
            N = [-1;0];
        elseif  x1(i)==1
            N = [1;0];  
        elseif x2(i)==1
            N = [0;1];
        elseif x2(i)==0
            N = [0;-1];     
        end    
        val(i,:) = sigma(i,:)*N;
    end 
end

% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.