%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Square domain with exact solution %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function data = SquareExact2
    exSolKnown = 1;
    [c4n,n4e,n4sDb,n4sNb] = loadGeometry('SquareNb2',1);
    data = {c4n,n4e,n4sDb,n4sNb,@f,@u4Db,@g,exSolKnown,...
                    @uExact,@graduExact};
end

%% PROBLEM INPUT DATA
% Right-hand side function
function val = f(x)
    val = - 2*(x(:,2)-0.5*x(:,2).^2) +x(:,1).^2;
end

% Dirichlet boundary data
function val = u4Db(x)
    val = uExact(x);
end

% Exact solution
function val = uExact(x)
    val = x(:,1).^2.*(x(:,2)-0.5*x(:,2).^2);
end

% Gradient of exact solution
function val = graduExact(x)
    val = [2*x(:,1).*(x(:,2)-0.5*x(:,2).^2),...
            x(:,1).^2.*(ones(size(x(:,2)))- x(:,2))];
end

% Neumann boundary data
function val = g(x)
    x1 = x(:,1);
    x2 = x(:,2);
    sigma = graduExact(x);
    for i=1:(size(x,1))    
        if x1(i)==0
            N = [-1;0];
        elseif  x1(i)==1
            N = [1;0];  
        elseif x2(i)==1
            N = [0;1];
        elseif x2(i)==0
            N = [0;-1];     
        end    
        val(i,:) = sigma(i,:)*N;
    end 
end

% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.