function [eta4e,error4eV] = estimatePrimalDPG(c4n,n4e,f,vApprox)
% Function to compute estimator and certain parts of error for primal dPG method
% Input:  c4n      coordinates for the nodes of the mesh
%         n4e      nodes for the elements of the mesh
%         f	       right hand side function
%         vApprox  piecewise P_1 approximation of residual
% Output: eta4e    error estimator for each element
%         error4eV error component of v in H^1 norm for each element

	%% Compute mesh quantities
    mid4e = computeMid4e(c4n,n4e);
    area4e = computeArea4e(c4n,n4e);
    
    % Prepare vector for gradients of vApprox
    error4eV = zeros(size(n4e,1),1);
    
    degree = 12;
    
    % Compute integral of f^2 on each element                           
    intFSq4e = integrate(c4n,n4e,@(n4p,Gpts4p,Gpts4ref)sum(f(Gpts4p).^2,2),degree,area4e);
    
    %% Compute H^1 error between vApprox and v=0
%     parfor curElem = 1:nrElems
    parfor curElem = 1:size(n4e,1)
        % Compute the nodes of the current element in global enumeration
        nodes = n4e(curElem,:);
        % Compute the coordinates of the nodes of the current element
        coords = c4n(nodes,:);
        % Compute the gradients of the barycentric coordinates of the
        % current element
        grads = [1,1,1;coords'] \ [0,0;eye(2)];
        vCurElem = vApprox((3*(curElem-1)+1): 3*curElem);
        % Compute H^1 norm of vApprox
        error4eV(curElem) = area4e(curElem) * vCurElem'*(grads * grads' + eye(3)/3)*vCurElem;  
    end

    %% Compute total error estimator 
    eta4e = area4e.*intFSq4e + error4eV;
end



% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.