function localOszillations = calculateF0(c4n,n4e,F,f)
% Function to compute local oscillation vectors in weighted LS
% Input:  c4n      coordinates for the nodes of the mesh
%         n4e      nodes for the elements of the mesh
%         F	       tag for choice F_0 in weighted LS
% Output: localMatrices      2-vector for each element

    %% Distinguish cases for different possibilities of F
    % Case for F = zero vector
    if strcmp(F,'zero')
        localOszillations = zeros(size(n4e,1),2);
    % Case for F = Pi_0(f*(identity-mid))
    elseif strcmp(F,'H0')
        localOszillations = calculateF1HZero(c4n,n4e,f);
    else 
        disp('F_0 not defined.'); 
    end
end

%% Calculate vector with Pi_0(f*(identity-mid)) on each element
function localOszillations = calculateF1HZero(c4n,n4e,f)
    % Compute mesh quantities
    area4e = computeArea4e(c4n,n4e);
    mid4e = computeMid4e(c4n, n4e);  
    
    % Compute integral of f on each element
    degreeF = 20; % degree of numerical integration
    intF4e = integrate(c4n, n4e, @(n4p,Gpts4p,Gpts4ref)f(Gpts4p), ...
                   degreeF, area4e);
    
    % Compute integral of f*identity mapping in each component           
    comp1 = @(z)(z(:,1)); % first component of identity mapping
    comp2 = @(z)(z(:,2)); % second component of identity mapping
    % Compute integral of f*identity mapping in first component 
    intFIdentity4eFirst = integrate(c4n, n4e, @(n4p,Gpts4p,Gpts4ref)(f(Gpts4p).*comp1(Gpts4p)), ...
                   degreeF, area4e);
    % Compute integral of f*identity mapping in second component 
    intFIdentity4eSecond = integrate(c4n, n4e, @(n4p,Gpts4p,Gpts4ref)(f(Gpts4p).*comp2(Gpts4p)), ...
                   degreeF, area4e); 
    % Piece together integral of f*identity mapping
    intFIdentity4e = [intFIdentity4eFirst intFIdentity4eSecond];
    
    % Compute Pi_0(f*(identity-mid)) = Pi_0(f*identity) -  Pi_0(f)*mid
    localOszillations = (intFIdentity4e - repmat(intF4e,1,2).*mid4e)./repmat(area4e,1,2);
end


% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.