function localMatrices = calculateM0(c4n,n4e,M)
% Function to compute local weight matrices in weighted LS
% Input:  c4n      coordinates for the nodes of the mesh
%         n4e      nodes for the elements of the mesh
%         M	       tag for choice of M_0 in weighted LS
% Output: localMatrices      2x2 SPD matrix for each element

    %% Distinguish cases for different possibilities of M
    % Case for M_0 = identity matrix
    if strcmp(M,'identity')
        localMatrices = repmat(eye(2),1,1,size(n4e,1));
    % Case for M_0 = identity matrix + Pi_0((identity-mid)^2)
    elseif strcmp(M,'identitySZero')
        localMatrices = repmat(eye(2),1,1,size(n4e,1)) + calculateM0SZero(c4n,n4e);
    % Case for M_0 = 2* identity matrix + Pi_0((identity-mid)^2)
    elseif strcmp(M,'twoidentitySZero')
        localMatrices = 2 * repmat(eye(2),1,1,size(n4e,1)) + calculateM0SZero(c4n,n4e);
    else 
        disp('M_0 not defined.'); 
    end
end

%% Calculate vector with Pi_0((identity-mid)^2) on each element
function localMatrices = calculateM0SZero(c4n,n4e)
    % Compute mesh quantities
    nrElems = size(n4e,1);
    % Prepare output vector
    localMatrices = zeros(2,2,nrElems);
%     parfor curElem = 1:nrElems   
    parfor curElem = 1:nrElems   
        % Compute the nodes of the current element in global enumeration
        nodes = n4e(curElem,:);
        % Compute the coordinates of the nodes of the current element
        coords = c4n(nodes,:);
        % Compute a matrix containing the vectors of the three edges
        edges = coords - coords([2,3,1],:);
        % Compute Matrix Pi_0((identity-mid)^2) for current element
        localMatrices(:,:,curElem) = 1/36*edges'*edges;
    end
end

% Copyright 2018 Friederike Hellwig
%
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.